function [V, W, iter, relChange, newshift] = IRKA(E, A, B, C, shift, tol, maxIter)
%
% function [V, W] = IRKA(E, A, B, C, shift, tol, maxIter)
%
% Input: E, A, B, C, shift, tol
%   where (E, A, B, C) -- state space realization
%         shift        -- initial selection of shifts
%         tol          -- stop tolerance
%         maxIter      -- maximum number of iterations
%
% Output: V, W
%   where V and W are projection matrices such that the state space
%         realization of the reduced system (Er, Ar, Br, Cr) are
%         computed as
%               Er = W^T * E* V,  Ar = W^T * A * V
%               Br = W^T * B,     Cr = C * V
%
% Note: + This function implements a generalization of the IRKA method
%         to descriptor systems. The IRKA is proposed by Gugercin et al.
%       + Shifts will be sorted by real parts and then imaginary parts if
%         their real parts are too close to call wrt a threshold.
%       + This function works with SISO systems.
%
% Ryan Nong
% June 19, 2008
%

k = length(shift);
shift

% sort initial shifts
threshold = 1e-10;
[trash, ord1] = sort(real(shift));
shift = shift(ord1);
i = 1;
while i < length(shift)
    if imag(shift(i)) == 0 % shift(i) is real
        if imag(shift(i + 1)) == 0 % shift(i + 1) is real
            i = i + 1;
        else % shift(i + 1) is complex
            i = i + 2;
        end
    else % shift(i) is complex
        if imag(shift(i + 1)) == 0 % shift(i + 1) is real
            if (real(shift(i + 1)) - real(shift(i))) < threshold
                temp = shift(i - 1);
                shift(i - 1) = shift(i + 1);
                shift(i + 1) = temp;
            end
            i = i + 1;
        else % shift(i + 1) is complex
            if real(shift(i)) == real(shift(i + 1)) % beginning of the pack
                i = i + 1;
            else % no longer the beginning of the pack
                if ((real(shift(i + 1)) - real(shift(i))) < threshold) && ...
                   (abs(imag(shift(i))) - abs(imag(shift(i + 1))) > threshold)
                    temp1 = shift(i - 1);
                    temp2 = shift(i);
                    shift(i - 1) = shift(i + 1);
                    shift(i) = shift(i + 2);
                    shift(i + 1) = temp1;
                    shift(i + 2) = temp2;
                end
                i = i + 2;
            end
        end
    end
end

%% initialize IRKA
j = 1;
V = [];
W = [];

while j <= k
    V = [V (shift(j) * E - A) \ B];
    W = [W (shift(j) * E' - A') \ C'];
    
    % if shift is complex
    if imag(shift(j)) ~= 0
        V(:, j:j + 1) = [real(V(:, j)) imag(V(:, j))];
        W(:, j:j + 1) = [real(W(:, j)) imag(W(:, j))];
        j = j + 1;
    end
    
    j = j + 1;
end

%[V, W] = biorthoArnoldi1(V, W);
[W, junk] = qr(W, 0);
[V, junk] = qr(V, 0);
WV = W' * V;
condWV = cond(WV)
Er = WV \ (W' * E * V);
%pause
%Er = eye(k);
Ar = WV \ (W' * A * V);

newshift = -eig(Ar, Er);
%newshift = computeNewShift(Ar, Er, shift, A, E, B, C, V, W); %Newton's

% replace NaN and Inf by random numbers
nanCheck = isnan(newshift);
infCheck = isinf(newshift);
if (sum(nanCheck) ~= 0 | sum(infCheck) ~= 0)
    display('change in shift because of existence of Nan/Inf');
    %pause;
end

nanArr = find(nanCheck);
for j = 1:length(nanArr)
    %newshift(find(nanCheck)) = abs(randn(1));
    newshift(nanArr(j)) = abs(randn(1));
end
infArr = find(infCheck);
for j = 1:length(infArr)
    %newshift(find(infCheck)) = abs(randn(1));
    newshift(infArr(j)) = abs(randn(1));
end
    
% sort new shifts
[trash, ord1] = sort(real(newshift));
newshift = newshift(ord1);
i = 1;
while i < length(newshift)
    if imag(newshift(i)) == 0 % shift(i) is real
        if imag(newshift(i + 1)) == 0 % shift(i + 1) is real
            i = i + 1;
        else % shift(i + 1) is complex
            i = i + 2;
        end
    else % shift(i) is complex
        if imag(newshift(i + 1)) == 0 % shift(i + 1) is real
            if (real(newshift(i + 1)) - real(newshift(i))) < threshold
                temp = newshift(i - 1);
                newshift(i - 1) = newshift(i + 1);
                newshift(i + 1) = temp;
            end
            i = i + 1;
        else % shift(i + 1) is complex
            if real(newshift(i)) == real(newshift(i + 1)) % beginning of the pack
                i = i + 1;
            else % no longer the beginning of the pack
                if ((real(newshift(i + 1)) - real(newshift(i))) < threshold) && ...
                   (abs(imag(newshift(i))) - abs(imag(newshift(i + 1))) > threshold)
                    temp1 = newshift(i - 1);
                    temp2 = newshift(i);
                    newshift(i - 1) = newshift(i + 1);
                    newshift(i) = newshift(i + 2);
                    newshift(i + 1) = temp1;
                    newshift(i + 2) = temp2;
                end
                i = i + 2;
            end
        end
    end
end

rel = norm(newshift - shift) / norm(shift);
relChange = [rel];

%% start iteration
iter = 1;
figure(1000);
hold;
while (rel > tol && iter < maxIter)
    shift = newshift
    figure(1000);
    plot(real(shift), imag(shift), 'r.');
    
    % run IRKA
    j = 1;
    V = [];
    W = [];

    while j <= k
        V = [V (shift(j) * E - A) \ B];
        W = [W (shift(j) * E' - A') \ C'];
    
        % if shift is complex
        if imag(shift(j)) ~= 0
            V(:, j:j + 1) = [real(V(:, j)) imag(V(:, j))];
            W(:, j:j + 1) = [real(W(:, j)) imag(W(:, j))];
            j = j + 1;
        end
    
        j = j + 1;
    end

%    [V, W] = biorthoArnoldi1(V, W);
    [W, junk] = qr(W, 0);
    [V, junk] = qr(V, 0);
    WV = W' * V;
condWV = cond(WV)
    Er = WV \ (W' * E * V);
%    pause
    %Er = eye(k);
    Ar = WV \ (W' * A * V);

    newshift = -eig(Ar, Er);
    
    % replace NaN and Inf by random numbers
    nanCheck = isnan(newshift);
    infCheck = isinf(newshift);
    if (sum(nanCheck) ~= 0 | sum(infCheck) ~= 0)
        display('change in shift because of existence of Nan/Inf');
        %pause;
    end

    nanArr = find(nanCheck);
    for j = 1:length(nanArr)
        %newshift(find(nanCheck)) = abs(randn(1));
        newshift(nanArr(j)) = abs(randn(1));
    end
    infArr = find(infCheck);
    for j = 1:length(infArr)
        %newshift(find(infCheck)) = abs(randn(1));
        newshift(infArr(j)) = abs(randn(1));
    end

    % sort new shifts
    [trash, ord1] = sort(real(newshift));
    newshift = newshift(ord1);
    i = 1;
    while i < length(newshift)
        if imag(newshift(i)) == 0 % shift(i) is real
            if imag(newshift(i + 1)) == 0 % shift(i + 1) is real
                i = i + 1;
            else % shift(i + 1) is complex
                i = i + 2;
            end
        else % shift(i) is complex
            if imag(newshift(i + 1)) == 0 % shift(i + 1) is real
                if (real(newshift(i + 1)) - real(newshift(i))) < threshold
                    temp = newshift(i - 1);
                    newshift(i - 1) = newshift(i + 1);
                    newshift(i + 1) = temp;
                end
                i = i + 1;
            else % shift(i + 1) is complex
                if real(newshift(i)) == real(newshift(i + 1)) % beginning of the pack
                    i = i + 1;
                else % no longer the beginning of the pack
                    if ((real(newshift(i + 1)) - real(newshift(i))) < threshold) && ...
                       (abs(imag(newshift(i))) - abs(imag(newshift(i + 1))) > threshold)
                        temp1 = newshift(i - 1);
                        temp2 = newshift(i);
                        newshift(i - 1) = newshift(i + 1);
                        newshift(i) = newshift(i + 2);
                        newshift(i + 1) = temp1;
                        newshift(i + 2) = temp2;
                    end
                    i = i + 2;
                end
            end
        end
    end

    rel = norm(newshift - shift) / norm(shift);
    relChange = [relChange rel];
    
    iter = iter + 1
end
hold;

figure;
semilogy(1:iter, relChange, 'b+-');
title('Convergence History of Shifts')

